# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from google.cloud.backupdr_v1 import gapic_version as package_version

__version__ = package_version.__version__


from .services.backup_dr import BackupDRAsyncClient, BackupDRClient
from .types.backupdr import (
    CreateManagementServerRequest,
    DeleteManagementServerRequest,
    GetManagementServerRequest,
    ListManagementServersRequest,
    ListManagementServersResponse,
    ManagementServer,
    ManagementURI,
    NetworkConfig,
    OperationMetadata,
    WorkforceIdentityBasedManagementURI,
    WorkforceIdentityBasedOAuth2ClientID,
)
from .types.backupplan import (
    BackupPlan,
    BackupRule,
    BackupWindow,
    CreateBackupPlanRequest,
    DeleteBackupPlanRequest,
    GetBackupPlanRequest,
    ListBackupPlansRequest,
    ListBackupPlansResponse,
    StandardSchedule,
    WeekDayOfMonth,
)
from .types.backupplanassociation import (
    BackupPlanAssociation,
    CreateBackupPlanAssociationRequest,
    DeleteBackupPlanAssociationRequest,
    GetBackupPlanAssociationRequest,
    ListBackupPlanAssociationsRequest,
    ListBackupPlanAssociationsResponse,
    RuleConfigInfo,
    TriggerBackupRequest,
)
from .types.backupvault import (
    Backup,
    BackupApplianceBackupConfig,
    BackupApplianceLockInfo,
    BackupConfigInfo,
    BackupConfigState,
    BackupLock,
    BackupVault,
    BackupVaultView,
    BackupView,
    CreateBackupVaultRequest,
    DataSource,
    DataSourceBackupApplianceApplication,
    DataSourceGcpResource,
    DeleteBackupRequest,
    DeleteBackupVaultRequest,
    FetchUsableBackupVaultsRequest,
    FetchUsableBackupVaultsResponse,
    GcpBackupConfig,
    GcpResource,
    GetBackupRequest,
    GetBackupVaultRequest,
    GetDataSourceRequest,
    ListBackupsRequest,
    ListBackupsResponse,
    ListBackupVaultsRequest,
    ListBackupVaultsResponse,
    ListDataSourcesRequest,
    ListDataSourcesResponse,
    RestoreBackupRequest,
    RestoreBackupResponse,
    ServiceLockInfo,
    TargetResource,
    UpdateBackupRequest,
    UpdateBackupVaultRequest,
    UpdateDataSourceRequest,
)
from .types.backupvault_ba import BackupApplianceBackupProperties
from .types.backupvault_gce import (
    AcceleratorConfig,
    AccessConfig,
    AdvancedMachineFeatures,
    AliasIpRange,
    AllocationAffinity,
    AttachedDisk,
    ComputeInstanceBackupProperties,
    ComputeInstanceDataSourceProperties,
    ComputeInstanceRestoreProperties,
    ComputeInstanceTargetEnvironment,
    ConfidentialInstanceConfig,
    CustomerEncryptionKey,
    DisplayDevice,
    Entry,
    GuestOsFeature,
    InstanceParams,
    KeyRevocationActionType,
    Metadata,
    NetworkInterface,
    NetworkPerformanceConfig,
    Scheduling,
    SchedulingDuration,
    ServiceAccount,
    Tags,
)

__all__ = (
    "BackupDRAsyncClient",
    "AcceleratorConfig",
    "AccessConfig",
    "AdvancedMachineFeatures",
    "AliasIpRange",
    "AllocationAffinity",
    "AttachedDisk",
    "Backup",
    "BackupApplianceBackupConfig",
    "BackupApplianceBackupProperties",
    "BackupApplianceLockInfo",
    "BackupConfigInfo",
    "BackupConfigState",
    "BackupDRClient",
    "BackupLock",
    "BackupPlan",
    "BackupPlanAssociation",
    "BackupRule",
    "BackupVault",
    "BackupVaultView",
    "BackupView",
    "BackupWindow",
    "ComputeInstanceBackupProperties",
    "ComputeInstanceDataSourceProperties",
    "ComputeInstanceRestoreProperties",
    "ComputeInstanceTargetEnvironment",
    "ConfidentialInstanceConfig",
    "CreateBackupPlanAssociationRequest",
    "CreateBackupPlanRequest",
    "CreateBackupVaultRequest",
    "CreateManagementServerRequest",
    "CustomerEncryptionKey",
    "DataSource",
    "DataSourceBackupApplianceApplication",
    "DataSourceGcpResource",
    "DeleteBackupPlanAssociationRequest",
    "DeleteBackupPlanRequest",
    "DeleteBackupRequest",
    "DeleteBackupVaultRequest",
    "DeleteManagementServerRequest",
    "DisplayDevice",
    "Entry",
    "FetchUsableBackupVaultsRequest",
    "FetchUsableBackupVaultsResponse",
    "GcpBackupConfig",
    "GcpResource",
    "GetBackupPlanAssociationRequest",
    "GetBackupPlanRequest",
    "GetBackupRequest",
    "GetBackupVaultRequest",
    "GetDataSourceRequest",
    "GetManagementServerRequest",
    "GuestOsFeature",
    "InstanceParams",
    "KeyRevocationActionType",
    "ListBackupPlanAssociationsRequest",
    "ListBackupPlanAssociationsResponse",
    "ListBackupPlansRequest",
    "ListBackupPlansResponse",
    "ListBackupVaultsRequest",
    "ListBackupVaultsResponse",
    "ListBackupsRequest",
    "ListBackupsResponse",
    "ListDataSourcesRequest",
    "ListDataSourcesResponse",
    "ListManagementServersRequest",
    "ListManagementServersResponse",
    "ManagementServer",
    "ManagementURI",
    "Metadata",
    "NetworkConfig",
    "NetworkInterface",
    "NetworkPerformanceConfig",
    "OperationMetadata",
    "RestoreBackupRequest",
    "RestoreBackupResponse",
    "RuleConfigInfo",
    "Scheduling",
    "SchedulingDuration",
    "ServiceAccount",
    "ServiceLockInfo",
    "StandardSchedule",
    "Tags",
    "TargetResource",
    "TriggerBackupRequest",
    "UpdateBackupRequest",
    "UpdateBackupVaultRequest",
    "UpdateDataSourceRequest",
    "WeekDayOfMonth",
    "WorkforceIdentityBasedManagementURI",
    "WorkforceIdentityBasedOAuth2ClientID",
)
